/* eslint-disable no-console */
import express from 'express';
import bodyParser from 'body-parser';
import errorHandler from 'errorhandler';
import consoleStamp from 'console-stamp';
import methodOverride from 'method-override';
import fs from 'fs';
import morgan from 'morgan';
import path from 'path';
import chalk from 'chalk';
import moment from 'moment';
import fallback from 'express-history-api-fallback';
import subdomain from 'express-subdomain';
import config from '@orchestra.group/cyiot-configs/lib/http-server';
import { version } from './package';
// import * as globalConfig from '@orchestra.group/cyiot-configs';

const LOG_DATE_FORMAT = {
	MORGAN: 'YYYY-MM-DD HH:MM:ss',
	CONSOLE_STAMP: 'yyyy-mm-dd HH:MM:ss',
};

const { port } = config;

const app = express();
app.use(methodOverride());
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({
	extended: true,
}));
app.use(errorHandler({
	dumpExceptions: true,
	showStack: true,
}));

app.get('/is-alive', (req, res) => {
	return res.status(200).send(version);
});

morgan.token('formatted-date', () => moment().format(LOG_DATE_FORMAT.MORGAN));
consoleStamp(console, LOG_DATE_FORMAT.CONSOLE_STAMP);

const morganTemplate = `${chalk.dim('[:formatted-date]')} - ${chalk.green(':method')} ${chalk.magentaBright(':url')} ${chalk.bold.blue(':status')} - ${chalk.yellow(':response-time')} ms`;
app.use(morgan(morganTemplate));

// Verify public dir
const publicDir = path.resolve(`${__dirname}/public`);
if (!fs.existsSync(publicDir)){
	console.error('No public directory! Existing');
	process.exit(1);
}
app.use(express.static(publicDir));

// Register underlying static modules
const registerModule = module => {
	const moduleRouter = express.Router();
	const root = path.join(publicDir, `${module}`);
	moduleRouter.get('/', (req, res) => {
		res.sendFile(path.join(root, '/index.html'));
	});
	moduleRouter.use(fallback('index.html', { root }));
	// 	moduleRouter.get('/config', (req, res) => {
	// 		res.json(globalConfig[`${module}`]);
	// 	});
	app.use(subdomain(`${module}`, moduleRouter));
	app.use(subdomain(`${module}.*`, moduleRouter)); // support Multilevel TLD's, such as .co.uk
	console.log(`Registered module ${module}`);
};

app.get('/404', (req, res) => {
	res.sendFile(path.join(publicDir, '404.html'));
});

app.get('/500', (req, res) => {
	res.status(500).sendFile(path.join(publicDir, '500.html'));
});

// Register all folders under public
const modules = fs.readdirSync(publicDir).filter(f => fs.statSync(path.join(publicDir, f)).isDirectory());
modules.forEach(registerModule);

app.use((req, res) => {
	res.redirect('/500');
});

console.log(`Cyiot-HTTP-Server listening on ${port}`);
app.listen(port);
// app.listen(port, hostname);

